from setuptools import setup, find_packages
import sys, os

version = '1.0b1'
shortdesc ="Container for queryable Records."
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.txt')).read() 
longdesc += open(os.path.join(os.path.dirname(__file__), 'LICENSE.txt')).read()

setup(name='cornerstone.soup',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      classifiers=[
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Framework :: Zope2',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules'        
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='container data record catalog',
      author='BlueDynamics Alliance',
      author_email='dev@bluedynamics.com',
      url='http://dev.plone.org/collective/browser/cornerstone.soup',
      license='BSD',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['cornerstone'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'zope.catalog',
          'five.intid',
          #'ZODB',
          # -*- Extra requirements: -*
      ],
      extras_require={
      },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )